/**
* OLAT - Online Learning and Training<br>
* http://www.olat.org
* <p>
* Licensed under the Apache License, Version 2.0 (the "License"); <br>
* you may not use this file except in compliance with the License.<br>
* You may obtain a copy of the License at
* <p>
* http://www.apache.org/licenses/LICENSE-2.0
* <p>
* Unless required by applicable law or agreed to in writing,<br>
* software distributed under the License is distributed on an "AS IS" BASIS, <br>
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
* See the License for the specific language governing permissions and <br>
* limitations under the License.
* <p>
* Copyright (c) since 2004 at Multimedia- & E-Learning Services (MELS),<br>
* University of Zurich, Switzerland.
* <hr>
* <a href="http://www.openolat.org">
* OpenOLAT - Online Learning and Training</a><br>
* This file has been modified by the OpenOLAT community. Changes are licensed
* under the Apache 2.0 license as the original file.
*/
package org.olat.course.nodes;

import org.olat.course.run.scoring.ScoreEvaluation;
import org.olat.course.run.userview.UserCourseEnvironment;

/**
 * 
 * Description:<br>
 * Provides a course node with the possibility to retrive its last ScoreEvaluation.
 * 
 * <P>
 * Initial Date:  11.04.2008 <br>
 * @author Lavinia Dumitrescu
 */
public interface SelfAssessableCourseNode extends CourseNode {

	/**
	 * Provides the ScoreEvaluation for this course node.
	 * Returns null if no scoring stored yet (that is no selftest finished yet).
	 * @param userCourseEnv
	 * @return Returns the ScoreEvaluation.
	 */
	public ScoreEvaluation getUserScoreEvaluation(UserCourseEnvironment userCourseEnv);
	
	/**
	 * @param userCourseEnvironment
	 * @return The users attempts of this node
	 */
	public Integer getUserAttempts(UserCourseEnvironment userCourseEnvironment);
	
	/**
	 * Increments the users attempts for this node and this user + 1. 
	 * @param userCourseEnvironment
	 */
	public void incrementUserAttempts(UserCourseEnvironment userCourseEnvironment);
	
}
